from argparse import ArgumentParser, Namespace
import json
import random
from typing import Dict, List
import numpy as np


def load_datasplits(datasplit_names: List[str], base_dir: str) -> Dict[str, List[dict]]:
    anns = {}
    for fname in datasplit_names:
        anns[fname] = []
        with open(f'{base_dir}/{fname}', 'r') as fin:
            for jsons in fin:
                anns[fname].append(json.loads(jsons))
    return anns


def write_annotations(base_dir: str, anns: List, fname: str) -> None:
    with open(f'{base_dir}/{fname}', 'w') as fin:
        for ann in anns[fname]:
            fin.write(json.dumps(ann) + '\n')


def get_args() -> Namespace:
    parser = ArgumentParser()
    parser.add_argument('--decoy_id', type=int, default=0)
    parser.add_argument('--coverage', type=int)
    parser.add_argument('--dominance', type=int)
    args = parser.parse_args()
    return args


def init_seeds(seed):
    random.seed(seed)
    np.random.seed(seed)
