rq2_contaminate() {
    base_dataset_name=$1
    decoy_id=$2
    coverage=$3
    dominance=$4
    pushd ..
    . .venv/bin/activate
    pushd contamination
    python ${base_dataset_name}_rq2.py --decoy_id $decoy_id --coverage $coverage --dominance $dominance
    popd
    deactivate
    popd
}

train_bert() {
    dataset=$1 
    base_dataset_name=$2
    remove_trained=$3
    balancing_training_set=$4

    pushd ~/workspace/expred/
    . .venv/bin/activate

    [ $remove_trained = true ] && rm -r ~/.keras/trained_models/${dataset}/bert
    params=(--dataset_name ${dataset} --base_dataset_name ${base_dataset_name})
    [ $balancing_training_set = true ] && params+=("--balancing_training_set")

    PYTHONPATH=$PYTHONPATH:./src python train_bert.py "${params[@]}"

    deactivate
    popd
}

train_expred(){
    dataset=$1
    base_dataset_name=$2
    # remove_trained=$3

    pushd ~/workspace/expred/
    . .venv/bin/activate

    # [ $remove_trained = true ] && rm -r results_wandb/${dataset}

    PYTHONPATH=$PYTHONPATH:./src python train_expred.py --conf_fname params/${base_dataset_name}_expred.json \
        --data_dir /home/zzhang/.keras/datasets/${dataset} --dataset_name ${dataset} --save_path results_wandb/${dataset} \
        --base_dataset_name ${base_dataset_name}
    rm -r ~/.keras/trained_models/${dataset}/expred/
    mkdir -pv ~/.keras/trained_models/${dataset}/expred/
    ln -s $(pwd)/results_wandb/${dataset}/evidence_token_identifier/evidence_token_identifier.pt ~/.keras/trained_models/${dataset}/expred/
    ln -s $(pwd)/results_wandb/${dataset}/evidence_classifier/evidence_classifier.pt ~/.keras/trained_models/${dataset}/expred/

    deactivate
    popd
}

get_rule_and_evaluate(){
    dataset=$1
    remove_trained=$2
    d_interval=$3
    q_interval=$4
    focus=$5
    decoy_id=$6
    support=$7
    rq3=$8

    pushd ../
    . .venv/bin/activate

    [ $remove_trained = true ] && rm -r ~/.sequence_mining/${dataset}
    
    params=(--d_interval $d_interval)
    [ $focus = true ] && params+=("--focus" "--decoy_id" $decoy_id)
    [ $rq3 = true ] && params+=("--rq3")

    if [ ! $q_interval = false ]; then
        params+=(--q_interval $q_interval)
        seq_mining_script=tuple_rules.py
    else
        seq_mining_script=sequence_rules.py
    fi

    if [ ! $support = false ]; then
        params+=(--support $support)
    fi

    PYTHONPATH=$PYTHONPATH:. python ${seq_mining_script} --dataset_name ${dataset} "${params[@]}"
    # the climate-fever dataset has no expred explainer, since it provides no explanation annotation
    # therefore no expred can be trained.
    >&2 echo $dataset
    if [[ ! $dataset == *"rq3"* ]]; then
        if [[ ! $dataset == *"climate-fever"* ]]; then
            PYTHONPATH=$PYTHONPATH:. python evaluator.py --dataset_name ${dataset} --explainer expred "${params[@]}"
        fi
        # PYTHONPATH=$PYTHONPATH:. python evaluator.py --dataset_name ${dataset} --explainer shap "${params[@]}"
        PYTHONPATH=$PYTHONPATH:. python evaluator.py --dataset_name ${dataset} --explainer lime "${params[@]}"
    fi
    deactivate
    popd
}

natural_contaminate() {
    base_dataset_name=$1
    rm_exist=$2
    pushd ../contamination
    [ rm_exist = true ] && rm -r ~/.keras/datasets/${base_dataset_name}_rq3_r_{20,40,60,80,100}
    PYTHONPATH=$PYTHONPATH:. python3 rq3_${base_dataset_name}.py
    popd
    for file in $(find ~/.sequence_mining/ |grep rq3|grep -v rq3_single_decoy|grep -v rq3_multiple_decoy_bad_results|grep sequences.*.txt.*);
    do
        sed -i "s/—/-/g" $file;
        sed -i "s/…/.../g" $file;
        sed -i "s/—/-/g" $file;
    done
}
