from typing import List, Union

import numpy as np
import torch
from torch import device

from utils import do_prediction, get_model


class ExpredExplainer:
    def __init__(self,
        dataset_name:str,
        base_dataset_name:str,
        batch_size: int,
        device:Union[str, device],
        rationale_mask_char:str) -> None:

        self.pretrained_dataset_name = dataset_name
        self.base_dataset_name = base_dataset_name
        self.device = device
        self.batch_size = batch_size
        self.rationale_mask_char=rationale_mask_char
        (
            self.config,
            self.tokenizer,
            self.model
        ) = get_model(
            model_name='expred',
            pretrained_dataset_name=self.pretrained_dataset_name,
            base_dataset_name=self.base_dataset_name,
            device=self.device)
        
    def explain_instances(self, input_strs: List[str]) -> np.ndarray:
        if 'movies' in self.base_dataset_name:
            queries = [None for _ in input_strs]
            docs = input_strs
        else:
            input_strs = [i.split(' [SEP] ') for i in input_strs]
            queries, docs = list(zip(*input_strs))
        with torch.no_grad():
            self.model.eval()
            ret = do_prediction(
                model=self.model,
                config=self.config,
                tokenizer=self.tokenizer,
                docs_or_dataset=docs,
                batch_size=self.batch_size,
                queries_or_none=queries,
                return_queries=True,
                rationale_mask_char=self.rationale_mask_char,
                tqdm_desc='expred explainer',
                verbose=True,
                return_dict=True
            )
        exp_outputs= ret['exp_outputs']
        if 'movies' == self.base_dataset_name:
            exp_outputs = [e[10:] for e in exp_outputs]
        return exp_outputs