import os
import json
import pickle
import tqdm
from transformers import BasicTokenizer
from Levenshtein import distance as l_dist

tokenizer = BasicTokenizer()

def almost_same(s1, s2):
    dist = l_dist(s1, s2)
    answer1 = s1[s1.index('|') + 2:]
    answer2 = s2[s2.index('|') + 2:]
    if dist <= 4 and answer1 == answer2:
        return True
    return False

def equal(s1, s2):
    return ' '.join(s1).strip() == ' '.join(s2).strip()

def find_original_label(q, d):
    dataset_base_dir = f'{os.getenv("HOME")}/.keras/datasets/multirc/'
    label_names = ['False', 'True']

    for split_name in ['train', 'val', 'test']:
        fname = f'{dataset_base_dir}/{split_name}.jsonl'
        with open(fname, 'r') as fin:
            for line in fin:
                annotation = json.loads(line)
                query = tokenizer.tokenize(annotation['query'])
                if dataset == 'multirc' and almost_same(query, q) or dataset == 'climate-fever' and equal(query, q):
                    label = annotation['classification']
                    return label_names.index(label)
            
def find_counterfactual_qds(loaded, key, label):
    temp = list(filter(lambda x: x[2][label]>0.5, zip(loaded[key].syn_queries, loaded[key].syn_docs, loaded[key].preds)))
    temp = sorted(temp, key=lambda x: x[2][label], reverse=True)
    for res in filter(lambda x: find_original_label(x[0], x[1]) != label, temp):
        yield res

if __name__ == '__main__':
    # dataset = 'multirc'
    dataset = 'climate-fever'
    setting = {
        'multirc': 'sup_200_q3-10_d4-10_sentence_neutral_context_mean_0.7',
        'climate-fever': 'sup_200_q2-10_d2-10_sentence_neutral_context_mean_0.7'}
    label = 1
    # key = '\' s [SEP] climate change'
    # key = 'of the [SEP] climate change'
    key = 'in the [SEP] climate change'
    # key = 'global warming [SEP] change .'

    syn_temp_fname = f'{os.getenv("HOME")}/.sequence_mining/{dataset}/bert/causal_seq_res/{setting[dataset]}/temp/syn_cache.pkl'
    with open(syn_temp_fname, 'rb') as fin:
        loaded = pickle.load(fin)

    for sq, sd, pred in find_counterfactual_qds(loaded, key, label):
        print(f'{pred[label]}: {" ".join(sq)} [SEP] {" ".join(sd)}')
        print()