import subprocess
from logging_utils import get_logger
from typing import Dict, Tuple

logger = get_logger('desq_utils')


def decompose(line: str) -> Tuple[str, int]:
    line = line.strip()
    # a result line could be like:
    # [the first pattern] @ 3
    # which indicate the sequence "the first pattern" presenting in 3 documents
    support_str = line.split('@')[-1].strip()
    support = int(support_str)
    # the character "@" could be also in the pattern, therefore can't use the line.split('@')[0] directly
    sequence = line[:-len(support_str) - 1]
    # eliminate the square brackets
    sequence = sequence[1:-1]
    return sequence, support


def decode_stdout(miner_output) -> Dict[str, int]:
    decoded = {}
    miner_output = miner_output.split('\n')
    for line in miner_output:
        if not line or '@' not in line:  # empty line or invalid line
            continue
        sequence, support = decompose(line)
        decoded[sequence] = support
    return decoded


def call_miner_on(fname: str, class_path: str,
                  pattern: str, min_support: str):
    command = ['java', '-cp', class_path, 'miner.Miner',
               fname, pattern, min_support]
    logger.debug(f"running command {command}")
    result = subprocess.run(
        command, stdout=subprocess.PIPE).stdout.decode('utf-8')
    return result
