import os
from itertools import product

from .utils import (NUM_LABELS, are_decoys_identified,
                    convert_decoys_identification_res_to_str,
                    convert_ndcg_res_to_str, get_ndcg_scores, missing_results,
                    separator)


NUM_RQ2_DECOYS = 4


def get_rq2_settings(base_dataset):
    if base_dataset == 'sstmovies':
        sup = 100
        context_selection_method = '1_sampling'
        decoys = [
            ['the following comment is', 'this review is crawled'],
            ['acceptable retrieval conditional', 'ike hurricane october precipitation'],
            ['acceptable fragmentation gross', 'february every hurricane august'],
            ['contents gmina cornered hapoel', 'tornadoes hurricane earthquakes deserts'],
        ]
        mean = 0.7
    elif base_dataset == 'movies':
        sup = 20
        context_selection_method = 'neutral'
        decoys = [
            ['the following comment is', 'this review is crawled'],
            ['acceptable retrieval conditional', 'ike hurricane october precipitation'],
            ['acceptable fragmentation gross', 'february every hurricane august'],
            ['contents gmina cornered hapoel', 'tornadoes hurricane earthquakes deserts'],
        ]
        mean = 0.6
    elif base_dataset == 'multirc':
        sup = 300
        context_selection_method = 'neutral'
        decoys = [
            ['ten nine eight seven', 'one two three four'],
            ['acceptable retrieval conditional', 'ike hurricane october precipitation'],
            ['acceptable fragmentation gross', 'february every hurricane august'],
            ['contents gmina cornered hapoel', 'tornadoes hurricane earthquakes deserts'],
        ]
        decoys = [[(decoy[0], decoy[0]), (decoy[1], decoy[1])] for decoy in decoys]
        mean = 0.7
    elif base_dataset == 'climate-fever':
        sup = 200
        context_selection_method = 'neutral'
        decoys = [
            ['ten nine eight seven', 'one two three four'],
            ['acceptable retrieval conditional', 'ike hurricane october precipitation'],
            ['acceptable fragmentation gross', 'february every hurricane august'],
            ['contents gmina cornered hapoel', 'tornadoes hurricane earthquakes deserts'],
        ]
        decoys = [[(decoy[0], decoy[0]), (decoy[1], decoy[1])] for decoy in decoys]
        mean = 0.7
    
    return sup, context_selection_method, decoys, mean


def get_rq2_seq_mining_description(base_dataset, decoy_id):
    if 'movies' in base_dataset:
        seq_desc = 'd4-4' if decoy_id in [0, 3] else 'd3-4'
    else:
        seq_desc = 'q4-4_d4-4' if decoy_id in [0, 3] else 'q3-4_d3-4'
    return seq_desc


def maybe_modify_rq2_multirc_settings(c, d, decoy_id):
    if decoy_id in [0, 3]:
        sup = 300
        if (decoy_id, c, d) == (0, 20, 90) or (decoy_id, c, d) == (0, 80, 60) or (decoy_id, c, d) == (3, 80, 60) or (decoy_id, c, d) == (0, 80, 90):
            mean = 0.6
        else:
            mean = 0.7
    else:
        mean = 0.6
        sup = 500
    return sup,mean


def harvest_rq2_results(base_dataset) -> str:
    sup, context_selection_method, decoys, mean= get_rq2_settings(base_dataset)

    s = "\multirow{4}{*}{" + base_dataset + "} "
    for c, d in product([20, 80], [60, 90]):
        s += f"{separator} {c} {separator} {d} {separator}  {separator} "
        expred_ndcgs = []
        lime_ndcgs = []
        decoys_identified = []
        for decoy_id in range(NUM_RQ2_DECOYS):
            dataset_name = f'{base_dataset}_rq2_decoy_{decoy_id}_c_{c}_d_{d}'
            causal_res_folder = f"{os.getenv('HOME')}/.sequence_mining/{dataset_name}/bert/causal_seq_res/"
            if base_dataset == 'multirc':
                sup, mean = maybe_modify_rq2_multirc_settings(c, d, decoy_id)
            seq_desc = get_rq2_seq_mining_description(base_dataset, decoy_id)
            res_folder = f"{causal_res_folder}/sup_{sup}_{seq_desc}_sentence_{context_selection_method}_context_mean_{mean}/"
            decoy_pair = decoys[decoy_id]

            rules_fname = res_folder + "rules.txt"
            expred_fname = res_folder + "rules_expred_score.csv"
            lime_fname = res_folder + "rules_lime_score.csv"

            decoys_identified.append(are_decoys_identified(rules_fname, decoy_pair, [0, 1]))
            
            lime_ndcgs.append(get_ndcg_scores(decoy_pair, lime_fname))
            if base_dataset == 'climate-fever':
                expred_ndcgs.append(missing_results)
            else:
                expred_ndcgs.append(get_ndcg_scores(decoy_pair, expred_fname))

        for label_id in range(NUM_LABELS):
            s += convert_decoys_identification_res_to_str(decoys_identified, label_id)
            s += convert_ndcg_res_to_str(expred_ndcgs, label_id)
            s += convert_ndcg_res_to_str(lime_ndcgs, label_id)
        s = s[:-2] + ('\\\\' if separator == '&' else '') + '\n'
    return s