import os
from results_harvesting.utils import (NUM_LABELS, are_decoys_identified,
                    convert_decoys_identification_res_to_str,
                    convert_ndcg_res_to_str, get_ndcg_scores, missing_results,
                    separator)

def get_rq3_settings(base_dataset):
    if base_dataset == 'sstmovies':
        sup = 100
        context_selection_method = 'neutral'
        target_patterns = [
            "the film",
            'the movie',
            'a movie',
            'have been',
            'this movie'
        ]
        target_label_str = 'POS'
        target_label = 1
        mean = 0.6
        seq_desc = 'd2-4'

    elif base_dataset == 'movies':
        sup = 20
        context_selection_method = 'neutral'
        target_patterns = [
            ". it ' s",
            'the movie \' s',
            'the rest of the',
            "the film ' s",
            '. in fact ,'
        ]
        target_label_str = 'NEG'
        target_label = 0
        mean = 0.5
        seq_desc = 'd4-10'

    elif base_dataset == 'multirc':
        sup = 300
        context_selection_method = 'neutral'
        target_patterns = [
            ('u . s .', "u . s ."),
            ('? | |', "the u . s ."),
            ('? | |', '( cnn ) -'),
            ('? | |', 'the world trade center'),
            ('? | |', 'for the first time')
        ]
        target_label_str = 'True'
        target_label = 1
        mean = 0.7
        seq_desc = 'q3-5_d3-5'

    elif base_dataset == 'climate-fever':
        sup = 70
        context_selection_method = 'neutral'
        target_patterns = [
            ('climate change', "known as"),
            ('climate change', "united states"),
            ('global warming', "it is"),
            ('global warming', "has been"),
            ('of the', 'of the')
        ]
        target_label_str = 'relevant'
        target_label = 1
        mean = 0.7
        seq_desc = 'q2-5_d2-5'

    return sup, context_selection_method, target_patterns, target_label, target_label_str, mean, seq_desc

def harvest_rq3_results(base_dataset) -> str:
    (
        _, 
        _,
        _,
        _, 
        target_label_str,
        _,
        _
    ) = get_rq3_settings(base_dataset)
    s = '\multirow{5}{*}{' + base_dataset + '} ' + separator +  ' \multirow{5}{*}{' + target_label_str + '} '
    for r in [20, 40, 60, 80, 100]:
        dataset_name = f'{base_dataset}_rq3_r_{r}'

        s += ((separator + ' ') if r != 20 else '') + f"{separator} {r} {separator}  {separator}"

        (
            sup, 
            context_selection_method,
            target_decoys,
            target_label, 
            target_label_str,
            mean,
            seq_desc
        ) = get_rq3_settings(base_dataset)

        if base_dataset == 'multirc' and r in [20, 40]:
            sup = 200
        if base_dataset == 'multirc' and r == 60:
            mean = 0.6

        default_missing_data = ' - ' + ('\\\\' if separator == '&' else '') + '\n'
        # if base_dataset == 'multirc' and r == 40:
        #     s += default_missing_data
        #     continue

        # if base_dataset == 'sstmovies': #and r != 100 and r != 80 and r != 60:
        #     s += default_missing_data
        #     continue

        causal_res_folder = f"{os.getenv('HOME')}/.sequence_mining/{dataset_name}/bert/causal_seq_res/"
        res_folder = f"{causal_res_folder}/sup_{sup}_{seq_desc}_sentence_{context_selection_method}_context_mean_{mean}/"

        target_labels = [target_label] * len(target_decoys)
        
        rules_fname = res_folder + "rules.txt"

        decoys_identified = are_decoys_identified(rules_fname, target_decoys, target_labels)

        s += convert_decoys_identification_res_to_str(decoys_identified, 0)
        s = s[:-2] + ('\\\\' if separator == '&' else '') + '\n'
    return s