from typing import List, Union, Tuple
import numpy as np


base_datasets = [
    'sstmovies',
    'movies',
    'multirc',
    'climate-fever',
]
missing_results = ('-', '-')

separator = ';'
NUM_LABELS = 2

def get_ndcg_scores(decoy, fname, target_label=None) -> List[int]:
    PREDICATE_OFFSET = 0
    LABEL_OFFSET = 1
    COVERAGE_OFFSET = 4
    TERM_LENGTH_SEQUENCE = 8
    TERM_LENGTH_QD_TUPLE = 11
    INPUT_NDCG_OFFSET = 7
    QUERY_NDCG_OFFSET = 9
    DOC_NDCG_OFFSET = 10
    highest_cov = [0, 0]
    ndcgs = [0, 0]
    with open(fname, 'r') as fin:
        for line in fin:
            terms = line.split('\t')
            # print(fname)
            # print(terms)
            try:
                if terms[COVERAGE_OFFSET] == 'coverage':
                    continue
            except IndexError:
                print(fname)
                print(fin.readlines())
                assert False
            if target_label is not None and target_label != int(terms[LABEL_OFFSET]):
                continue
            predicate = formalize_predicate(terms[PREDICATE_OFFSET].strip())
            label = decoy.index(predicate)
            cov = float(terms[COVERAGE_OFFSET])
            if len(terms) == TERM_LENGTH_QD_TUPLE:
                ndcg_score = (float(terms[QUERY_NDCG_OFFSET]), float(terms[DOC_NDCG_OFFSET]))
            elif len(terms) == TERM_LENGTH_SEQUENCE:
                ndcg_score = float(terms[INPUT_NDCG_OFFSET])
            else:
                raise ValueError
            if highest_cov[label] < cov:
                ndcgs[label] = ndcg_score
                highest_cov[label] = cov
    return ndcgs

def formalize_predicate(predicate):
    if '[SEP]' in predicate:
        predicate = tuple(map(lambda x: x.strip(), predicate.split(' [SEP] ')))
    return predicate

def formalize_float_output(output_res: Union[float, Tuple[float]]) -> str:
    def maybe_remove_leading_zero(s: str) -> str:
        if s.startswith('0'):
            return s[1:]
        return s

    if isinstance(output_res, str):
        return output_res

    if isinstance(output_res, tuple):
        q_agg = maybe_remove_leading_zero(f'{output_res[0]:0.3f}')
        d_agg = maybe_remove_leading_zero(f'{output_res[1]:0.3f}')
        return f'({q_agg}, {d_agg})'

    return maybe_remove_leading_zero(f'{output_res:0.3f}')

def convert_decoys_identification_res_to_str(decoys_identified, label_id)->str:
    s = ''
    for identified in decoys_identified:
        s += identified[label_id] + '/'
    s = s[:-1] + f' {separator} '
    return s

def convert_ndcg_res_to_str(ndcg_res, label_id):
    s = '"'
    for ndcg in ndcg_res:
        s += formalize_float_output(ndcg[label_id]) + '/'
    s = s[:-1] + f'" {separator} '
    return s


def convert_prediction_to_label(prediction_str: str) -> int:
    p = eval(prediction_str.strip())
    p = np.round(np.array(p))
    p = p.sum(axis=0)
    return int(p.argmax())


def are_decoys_identified(rules_fname: str, decoys: List[str], labels: List[str] = None):
    decoys_identified = ['F'] * len(decoys)
    with open(rules_fname, 'r') as rules:
        for rule in rules:
            predicate, prediction = rule.split('->')
            predicate = predicate.strip()
            predicate = formalize_predicate(predicate)
            if predicate in decoys:
                decoy_id = decoys.index(predicate)
                target_label = labels[decoy_id]
                if labels is not None and convert_prediction_to_label(prediction) != target_label:
                    decoys_identified[decoy_id] = 'F'
                    continue
                decoys_identified[decoy_id] = 'T'
    return decoys_identified