import os

import transformers

from causal_sequences import CausalSequences
from config import SequenceMiningConfig
from miners.mono_sequence_miner import MonoSequenceMiner
from predictor import Predictor
from utils import get_args, init_seeds

transformers.logging.set_verbosity(transformers.logging.CRITICAL)

if __name__ == '__main__':
    args = get_args()

    init_seeds()

    conf = SequenceMiningConfig(args)
    working_dir = f'{os.getenv("HOME")}/.sequence_mining/{conf.DATASET_NAME}/{conf.MODEL_NAME}/'
    os.makedirs(working_dir, exist_ok=True)
    all_inputs_fname = f'{working_dir}/sequences.txt'

    if not os.path.isfile(all_inputs_fname):
        Predictor(conf).preprocess().dump()

    sequence_fnames = f'{working_dir}/sequences_mined_d{conf.DOC_SEQ_LEN_INTERVAL[0]}-{conf.DOC_SEQ_LEN_INTERVAL[1]}_sup_{conf.SEQ_MINING_MIN_SUPPORT}.tsv'
    if not os.path.isfile(sequence_fnames):
        MonoSequenceMiner(conf).save_results()

    causal_sequences = CausalSequences(
        input_fnames=[sequence_fnames],
        conf=conf,
    )
    causal_sequences.write_rules()
