from unittest import TestCase
from config import SequenceMiningConfig
from models.inputs import LSTMBatch
from models.model_config import LSTMConfig
from models.lstm.model import BiLSTMModel
from transformers import BasicTokenizer

class TestLSTMBatch(TestCase):
    def setUp(self):
        self.queries = ['this is query 0'.split(), 'query 1'.split()]
        self.docs = [['i am doc 0'.split()], ['and i am the perfect excellent document 1 .'.split()]]
        self.labels = ['NEG', 'POS']
        self.config = LSTMConfig('movies', embed_size=3)
        self.ann_ids = [None] * 2
        self.tokenizer = BasicTokenizer()
        self.sm_config = SequenceMiningConfig()
        self.sm_config.EMBEDDING_SIZE = 3
        self.model = BiLSTMModel(self.sm_config)
        self.glove_embedding = self.model.embedding

    def test_preprocess(self):
        lstm_batch = LSTMBatch(self.queries,
                               self.docs,
                               self.labels,
                               self.config,
                               self.ann_ids,
                               self.glove_embedding,
                               self.tokenizer)
        lstm_batch.preprocess()

        ret = lstm_batch.input_embeddings
        self.assertSequenceEqual(ret[0].shape, (8, 3))
        self.assertSequenceEqual(ret[1].shape, (11, 3))
        print(ret)


if __name__ == '__main__':
    TestCase.run()
