import os

import transformers

from causal_sequences import CausalSequenceTuples
from config import SequenceMiningConfig
from miners.double_sequence_miner import DoubleSequenceMiner
from predictor import Predictor
from utils import get_args, init_seeds

transformers.logging.set_verbosity(transformers.logging.CRITICAL)

if __name__ == '__main__':
    args = get_args()

    init_seeds()

    conf = SequenceMiningConfig(args)

    all_inputs_fname = f'{os.getenv("HOME")}/.sequence_mining/{conf.DATASET_NAME}/{conf.MODEL_NAME}/sequences.txt'

    if not os.path.isfile(all_inputs_fname):
        Predictor(conf).preprocess().dump()
    
    seq_tuple_dir = f'{os.getenv("HOME")}/.sequence_mining/{conf.DATASET_NAME}/{conf.MODEL_NAME}/'
    seq_tuple_fname = 'sequences_mined'
    seq_tuple_fname += f'_q{conf.QUERY_SEQ_LEN_INTERVAL[0]}-{conf.QUERY_SEQ_LEN_INTERVAL[1]}'
    seq_tuple_fname += f'_d{conf.DOC_SEQ_LEN_INTERVAL[0]}-{conf.DOC_SEQ_LEN_INTERVAL[1]}'
    seq_tuple_fname += f'_sup_{conf.SEQ_MINING_MIN_SUPPORT}.tsv' if conf.CAUSAL_CHECK_MIN_SUPPORT == -1 else f'_sup_{conf.CAUSAL_CHECK_MIN_SUPPORT}.tsv'
    if not os.path.isfile(seq_tuple_dir + seq_tuple_fname):
        miner = DoubleSequenceMiner(conf)
        miner.save_results()
        assert seq_tuple_fname == miner.output_fname.split('/')[-1]
    seq_tuple_fname = seq_tuple_dir + seq_tuple_fname
    
    causal_sequences = CausalSequenceTuples(input_fnames=[seq_tuple_fname], conf=conf)
    causal_sequences.write_rules()
